// -*-Java-*-
//
//  This file is part of WEEVIL.
//
//  Authors: See the file AUTHORS for full details.
//
//  Copyright (C) 2002-2006 University of Colorado
//
//  Weevil is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  Weevil is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
//  USA, or send email to serl@cs.colorado.edu.
//
package weevil;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.StringTokenizer;

/**
 * a generic virtual time workload controller
 * 
 * This class implements a generic virtual time controller which will control
 * the scheduling of the workload execution according to the workload file.
 * 
 */
public class WeevilSingleJVMJavaActor {

	/**
	 * the overall controller of the whole execution process
	 * 
	 * This method control the time to start the execution, and also read the
	 * workload file from the beginning to the end and call the corresponding
	 * action for each line of the workload file.
	 * 
	 * In your system specific client workload actor, you should call the
	 * run_simulation() method to start the whole process, and the
	 * stop_simulation() method to stop the whole process. For example,
	 * @throws IOException 
	 * @throws NumberFormatException 
	 * 
	 * @see doc-files/Actor.java
	 */

	public static void main(String[] args) throws NumberFormatException, IOException {
		// we receive the name of an argument file
		System.out.println("Args length: " + args.length);
		if (args.length < 1) {
			System.err
					.println("Usage: WeevilSingleJVMJavaActor <argument file>");
			System.err.print("Arguments were: ");
			for (int i = 0; i < args.length; i++) {
				System.err.print(args[i] + " ");
			}
			System.err.print("\n");
			System.exit(1);
		}
		BufferedReader in = new BufferedReader(new FileReader(args[0]));
		String line;
		while ((line = in.readLine()) != null) {
			StringTokenizer st = new StringTokenizer(line, " ");
			if (st.countTokens() >= 3) {				
				int time = Integer.parseInt(st.nextToken());
				String wkld = st.nextToken();
				String className = st.nextToken();
				String[] pargs = new String[1];
				pargs[0] = "";
				while(st.hasMoreTokens()){
					pargs[0] = pargs[0] + st.nextToken() + " ";
					//System.out.println(pargs[0]);
				}				
				Runnable p = new WeevilJavaActor(time, wkld, className, pargs);
				Thread thisThread = new Thread(p);
				thisThread.start();
//				 try {
//					 thisThread.join();			            
//			     } catch (InterruptedException e) {}
			}
		}
	}
}
